import numpy as np
from scipy.optimize import minimize_scalar


def wrap_to_pi(angle):
    if isinstance(angle, np.ndarray):
        angle[angle > np.pi] = angle[angle > np.pi] - 2*np.pi
        angle[angle < -np.pi] = angle[angle < -np.pi] + 2*np.pi
    else:
        if angle > np.pi:
            angle -= 2*np.pi
        elif angle < -np.pi:
            angle += 2*np.pi
    return angle


def distance_traj_segment(p0, p1, q0, q1):
    """
    A function finding the distance between two trajectory segements
    :param p0: the initial point of trajectory segment 1
    :param p1: the final point of trajectory segment 1
    :param q0: the initial point of trajectory segment 2
    :param q1: the final point of trajectory segment 2
    :return: distance
    """
    def sq_distance(t):
        p = p0 + t*(p1 - p0)
        q = q0 + t*(q1 - q0)
        return np.sum((p - q)**2)
    res = minimize_scalar(sq_distance, bounds=(0, 1), method='bounded')
    d = np.sqrt(res.fun)
    return d
