from pysimenv.core.base import SimObject
from pysimenv.core.systems import MultipleSystem


class SignalGenerator(SimObject):
    def __init__(self, shaping_fun):
        super(SignalGenerator, self).__init__()
        self.shaping_fun = shaping_fun

    def forward(self):
        return self.shaping_fun(self.sim_clock.time)


class FeedbackControl(MultipleSystem):
    def __init__(self, system, control):
        super(FeedbackControl, self).__init__()
        self.system = system
        self.control = control

        self.attach_sim_objects([system, control])

    # implement
    def forward(self, r=None):
        y = self.system.output
        u_fb = self.control.forward(y, r)
        self.system.forward(u_fb)


class Sequential(MultipleSystem):
    def __init__(self, obj_list):
        super(Sequential, self).__init__()
        assert len(obj_list) > 0, "[Sequential] Invalid obj_list!"
        self.obj_list = obj_list
        self.first_obj = obj_list[0]
        self.other_obj_list = obj_list[1:]

        self.attach_sim_objects(obj_list)

    # implement
    def forward(self, *args, **kwargs):
        out = self.first_obj.forward(*args, **kwargs)
        for obj in self.other_obj_list:
            out = obj.forward(out)
        return out


class FlatEarthEnv(object):
    grav_accel = 9.805

