from pysimenv.core.base import SimObject
from pysimenv.core.utils import Timer


class BaseFunction(object):
    def __init__(self, fun=None):
        self.fun = fun

    def forward(self, *args, **kwargs):
        assert self.fun is not None, "Define fun property or implement the forward method"
        return self.fun(*args, **kwargs)


class ZeroOrderHold(SimObject):
    def __init__(self, fun, time_interval):
        super(ZeroOrderHold, self).__init__()
        if isinstance(fun, SimObject):
            self.fun_object = fun
            self.fun = fun.forward
        elif isinstance(fun, BaseFunction):
            self.fun_object = fun
            self.fun = fun.forward
        else:
            self.fun_object = None
            self.fun = fun

        self.time_interval = time_interval
        self.timer = Timer(time_interval)
        self.output = None

    # override
    def attach_sim_clock(self, sim_clock):
        super(ZeroOrderHold, self).attach_sim_clock(sim_clock)
        if self.fun_object is not None:
            self.fun_object.attach_sim_clock(sim_clock)

        self.timer.attach_sim_clock(sim_clock)
        self.timer.turn_on()

    # override
    def attach_log_timer(self, log_timer):
        super(ZeroOrderHold, self).attach_log_timer(log_timer)
        if self.fun_object is not None:
            self.fun_object.attach_log_timer(log_timer)

    # override
    def reset(self):
        super(ZeroOrderHold, self).reset()
        if self.fun_object is not None:
            self.fun_object.reset()
        if self.timer.sim_clock is not None:
            self.timer.reset()

    # implement
    def forward(self, *args, **kwargs):
        assert self.sim_clock is not None, "[ZeroOrderHold] Attach a clock first!"
        self.timer.forward()
        if self.timer.is_event:
            self.output = self.fun(*args, **kwargs)
        return self.output
